from collections import namedtuple
import pickle
import os
import uuid

import pandas as pd

from dagster import types
from dagster.pandas import DataFrame

SomeTuple = namedtuple('SomeTuple', 'foo')


def get_unittest_path():
    base_dir = '/tmp/dagster/scratch/unittests/{uuid}'.format(uuid=str(uuid.uuid4()))
    if not os.path.exists(base_dir):
        os.makedirs(base_dir)
    return base_dir


def roundtrip_typed_value(value, dagster_type):
    full_path = get_unittest_path()

    dagster_type.serialize_value(full_path, value)
    return dagster_type.deserialize_value(full_path)


def test_basic_serialization_string():
    assert roundtrip_typed_value('foo', types.String)


def test_basic_serialization():
    assert roundtrip_typed_value(1, types.Int) == 1
    assert roundtrip_typed_value(True, types.Bool) is True
    assert roundtrip_typed_value({'bar': 'foo'}, types.Dict) == {'bar': 'foo'}


def test_basic_pandas():
    df = pd.DataFrame({'a': [1, 2], 'b': [3, 4]})
    out_df = roundtrip_typed_value(df, DataFrame)
    assert out_df.equals(df)



from collections import namedtuple

import pytest

from dagster import (
    ConfigDefinition,
    DagsterEvaluateValueError,
    DagsterInvalidDefinitionError,
    Field,
    PipelineContextDefinition,
    PipelineDefinition,
    SolidDefinition,
    types,
)

from dagster.core.types import (
    process_incoming_composite_value,
    ScopedConfigInfo,
)

from dagster.core.definitions import build_config_dict_type


def test_noop_config():
    assert ConfigDefinition(types.Any)


def test_int_field():
    config_def = ConfigDefinition.config_dict(
        'SingleRequiredInt',
        {
            'int_field': Field(types.Int),
        },
    )

    assert config_def.config_type.evaluate_value({'int_field': 1}) == {'int_field': 1}


def test_int_fails():
    config_def = ConfigDefinition.config_dict(
        'SingleRequiredInt', {
            'int_field': Field(types.Int),
        }
    )

    with pytest.raises(DagsterEvaluateValueError):
        config_def.config_type.evaluate_value({'int_field': 'fjkdj'})

    with pytest.raises(DagsterEvaluateValueError):
        config_def.config_type.evaluate_value({'int_field': True})


def test_default_arg():
    config_def = ConfigDefinition.config_dict(
        'TestDefaultArg', {
            'int_field': Field(types.Int, default_value=2, is_optional=True),
        }
    )

    assert config_def.config_type.evaluate_value({}) == {'int_field': 2}


def _single_required_string_config_dict():
    return ConfigDefinition.config_dict(
        'SingleRequiredField', {'string_field': Field(types.String)}
    )


def _multiple_required_fields_config_dict():
    return ConfigDefinition.config_dict(
        'MultipleRequiredFields', {
            'field_one': Field(types.String),
            'field_two': Field(types.String),
        }
    )


def _single_optional_string_config_dict():
    return ConfigDefinition.config_dict(
        'SingleOptionalString', {'optional_field': Field(types.String, is_optional=True)}
    )


def _single_optional_string_field_config_dict_with_default():
    optional_field_def = Field(
        types.String,
        is_optional=True,
        default_value='some_default',
    )
    return ConfigDefinition.config_dict(
        'SingleOptionalStringWithDefault',
        {'optional_field': optional_field_def},
    )


def _mixed_required_optional_string_config_dict_with_default():
    return ConfigDefinition.config_dict(
        'MixedRequired', {
            'optional_arg': Field(
                types.String,
                is_optional=True,
                default_value='some_default',
            ),
            'required_arg': Field(types.String, is_optional=False),
            'optional_arg_no_default': Field(types.String, is_optional=True),
        }
    )


def _validate(config_def, value):
    return config_def.config_type.evaluate_value(value)


def test_single_required_string_field_config_type():
    assert _validate(_single_required_string_config_dict(), {'string_field': 'value'}) == {
        'string_field': 'value'
    }
    assert _validate(_single_required_string_config_dict(), {'string_field': None}) == {
        'string_field': None
    }

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_required_string_config_dict(), {})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_required_string_config_dict(), {'extra': 'yup'})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_required_string_config_dict(), {'string_field': 'yupup', 'extra': 'yup'})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_required_string_config_dict(), {'string_field': 1})


def test_multiple_required_fields_passing():
    assert _validate(
        _multiple_required_fields_config_dict(),
        {
            'field_one': 'value_one',
            'field_two': 'value_two',
        },
    ) == {
        'field_one': 'value_one',
        'field_two': 'value_two',
    }

    assert _validate(
        _multiple_required_fields_config_dict(),
        {
            'field_one': 'value_one',
            'field_two': None,
        },
    ) == {
        'field_one': 'value_one',
        'field_two': None,
    }


def test_multiple_required_fields_failing():
    with pytest.raises(DagsterEvaluateValueError):
        _validate(_multiple_required_fields_config_dict(), {})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_multiple_required_fields_config_dict(), {'field_one': 'yup'})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_multiple_required_fields_config_dict(), {'field_one': 'yup', 'extra': 'yup'})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(
            _multiple_required_fields_config_dict(), {
                'field_one': 'value_one',
                'field_two': 2
            }
        )


def test_single_optional_field_passing():
    assert _validate(_single_optional_string_config_dict(), {'optional_field': 'value'}) == {
        'optional_field': 'value'
    }
    assert _validate(_single_optional_string_config_dict(), {}) == {}

    assert _validate(_single_optional_string_config_dict(), {'optional_field': None}) == {
        'optional_field': None
    }


def test_single_optional_field_failing():
    with pytest.raises(DagsterEvaluateValueError):

        _validate(_single_optional_string_config_dict(), {'optional_field': 1})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_optional_string_config_dict(), {'dlkjfalksdjflksaj': 1})


def test_single_optional_field_passing_with_default():
    assert _validate(_single_optional_string_field_config_dict_with_default(), {}) == {
        'optional_field': 'some_default'
    }

    assert _validate(
        _single_optional_string_field_config_dict_with_default(), {'optional_field': 'override'}
    ) == {
        'optional_field': 'override'
    }


def test_mixed_args_passing():
    assert _validate(
        _mixed_required_optional_string_config_dict_with_default(), {
            'optional_arg': 'value_one',
            'required_arg': 'value_two',
        }
    ) == {
        'optional_arg': 'value_one',
        'required_arg': 'value_two',
    }

    assert _validate(
        _mixed_required_optional_string_config_dict_with_default(), {
            'required_arg': 'value_two',
        }
    ) == {
        'optional_arg': 'some_default',
        'required_arg': 'value_two',
    }

    assert _validate(
        _mixed_required_optional_string_config_dict_with_default(), {
            'required_arg': 'value_two',
            'optional_arg_no_default': 'value_three',
        }
    ) == {
        'optional_arg': 'some_default',
        'required_arg': 'value_two',
        'optional_arg_no_default': 'value_three',
    }


def _single_nested_config():
    return ConfigDefinition(
        config_type=types.ConfigDictionary(
            'ParentType', {
                'nested':
                Field(
                    dagster_type=types.ConfigDictionary(
                        'NestedType',
                        {'int_field': Field(types.Int)},
                    )
                ),
            }
        )
    )


def _nested_optional_config_with_default():
    return ConfigDefinition(
        config_type=types.ConfigDictionary(
            'ParentType', {
                'nested':
                Field(
                    dagster_type=types.ConfigDictionary(
                        'NestedType',
                        {'int_field': Field(
                            types.Int,
                            is_optional=True,
                            default_value=3,
                        )}
                    )
                ),
            }
        )
    )


def _nested_optional_config_with_no_default():
    nested_type = types.ConfigDictionary(
        'NestedType',
        {
            'int_field': Field(
                types.Int,
                is_optional=True,
            ),
        },
    )
    return ConfigDefinition(
        config_type=types.ConfigDictionary(
            'ParentType',
            {'nested': Field(dagster_type=nested_type)},
        )
    )


def test_single_nested_config():
    assert _validate(_single_nested_config(), {'nested': {
        'int_field': 2
    }}) == {
        'nested': {
            'int_field': 2
        }
    }


def test_single_nested_config_failures():
    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_nested_config(), {'nested': 'dkjfdk'})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_nested_config(), {'nested': {'int_field': 'dkjfdk'}})

    with pytest.raises(DagsterEvaluateValueError):
        _validate(_single_nested_config(), {'nested': {'int_field': {'too_nested': 'dkjfdk'}}})


def test_nested_optional_with_default():
    assert _validate(_nested_optional_config_with_default(), {'nested': {
        'int_field': 2
    }}) == {
        'nested': {
            'int_field': 2
        }
    }

    assert _validate(_nested_optional_config_with_default(), {'nested': {}}) == {
        'nested': {
            'int_field': 3
        }
    }


def test_nested_optional_with_no_default():
    assert _validate(_nested_optional_config_with_no_default(), {'nested': {
        'int_field': 2
    }}) == {
        'nested': {
            'int_field': 2
        }
    }

    assert _validate(_nested_optional_config_with_no_default(), {'nested': {}}) == {'nested': {}}


CustomStructConfig = namedtuple('CustomStructConfig', 'foo bar')


class CustomStructConfigType(types.DagsterCompositeType):
    def __init__(self):
        super(CustomStructConfigType, self).__init__(
            'CustomStructConfigType',
            {
                'foo': Field(types.String),
                'bar': Field(types.Int),
            },
        )

    def evaluate_value(self, value):
        if value is not None and not isinstance(value, dict):
            raise DagsterEvaluateValueError('Incoming value for composite must be dict')

        return process_incoming_composite_value(
            self,
            value,
            lambda val: CustomStructConfig(foo=val['foo'], bar=val['bar']),
        )


def test_custom_composite_type():
    config_type = CustomStructConfigType()

    assert config_type.evaluate_value({
        'foo': 'some_string',
        'bar': 2
    }) == CustomStructConfig(
        foo='some_string', bar=2
    )

    with pytest.raises(DagsterEvaluateValueError):
        assert config_type.evaluate_value({
            'foo': 'some_string',
        })

    with pytest.raises(DagsterEvaluateValueError):
        assert config_type.evaluate_value({
            'bar': 'some_string',
        })

    with pytest.raises(DagsterEvaluateValueError):
        assert config_type.evaluate_value({
            'foo': 'some_string',
            'bar': 'not_an_int',
        })


def single_elem(ddict):
    return list(ddict.items())[0]


def test_build_config_dict_type():
    single_cd_type = build_config_dict_type(
        ['PipelineName', 'SingleField'],
        {'foo': types.Field(types.String)},
        ScopedConfigInfo(
            pipeline_def_name='pipeline_name',
            solid_def_name='single_field',
        ),
    )
    assert isinstance(single_cd_type, types.ConfigDictionary)
    assert single_cd_type.name == 'PipelineName.SingleField.ConfigDict'
    assert len(single_cd_type.field_dict) == 1
    foo_name, foo_field = single_elem(single_cd_type.field_dict)
    assert foo_name == 'foo'
    assert foo_field.dagster_type is types.String


def test_build_single_nested():
    def _assert_facts(single_nested):
        assert single_nested.name == 'PipelineName.Solid.SolidName.ConfigDict'
        assert set(single_nested.field_dict.keys()) == set(['foo', 'nested_dict'])

        assert single_nested.field_dict['nested_dict'].is_optional is False
        nested_field_type = single_nested.field_dict['nested_dict'].dagster_type

        assert isinstance(nested_field_type, types.ConfigDictionary)
        assert nested_field_type.name == 'PipelineName.Solid.SolidName.NestedDict.ConfigDict'
        assert nested_field_type.field_name_set == set(['bar'])

    old_style_config_def = ConfigDefinition(
        types.ConfigDictionary(
            'PipelineName.Solid.SolidName.ConfigDict',
            {
                'foo':
                types.Field(types.String),
                'nested_dict':
                types.Field(
                    types.ConfigDictionary(
                        'PipelineName.Solid.SolidName.NestedDict.ConfigDict',
                        {
                            'bar': types.Field(types.String),
                        },
                    ),
                ),
            },
        ),
    )

    _assert_facts(old_style_config_def.config_type)

    single_nested_manual = build_config_dict_type(
        ['PipelineName', 'Solid', 'SolidName'],
        {
            'foo': types.Field(types.String),
            'nested_dict': {
                'bar': types.Field(types.String),
            },
        },
        ScopedConfigInfo(
            pipeline_def_name='pipeline_name',
            solid_def_name='solid_name',
        ),
    )

    _assert_facts(single_nested_manual)

    nested_from_config_def = ConfigDefinition.solid_config_dict(
        'pipeline_name',
        'solid_name',
        {
            'foo': types.Field(types.String),
            'nested_dict': {
                'bar': types.Field(types.String),
            },
        },
    )

    _assert_facts(nested_from_config_def.config_type)


def test_build_double_nested():
    double_config_type = ConfigDefinition.context_config_dict(
        'some_pipeline',
        'some_context',
        {
            'level_one': {
                'level_two': {
                    'field': types.Field(types.String)
                }
            }
        },
    ).config_type

    assert double_config_type.name == 'SomePipeline.Context.SomeContext.ConfigDict'

    level_one_type = double_config_type.field_dict['level_one'].dagster_type

    assert isinstance(level_one_type, types.ConfigDictionary)
    assert level_one_type.name == 'SomePipeline.Context.SomeContext.LevelOne.ConfigDict'
    assert level_one_type.field_name_set == set(['level_two'])

    level_two_type = level_one_type.field_dict['level_two'].dagster_type

    assert level_two_type.name == 'SomePipeline.Context.SomeContext.LevelOne.LevelTwo.ConfigDict'
    assert level_two_type.field_name_set == set(['field'])


def test_build_optionality():
    optional_test_type = ConfigDefinition.solid_config_dict(
        'some_pipeline',
        'some_solid',
        {
            'required': {
                'value': types.Field(types.String),
            },
            'optional': {
                'value': types.Field(types.String, is_optional=True),
            }
        },
    ).config_type

    assert optional_test_type.field_dict['required'].is_optional is False
    assert optional_test_type.field_dict['optional'].is_optional is True


def test_pipeline_name_mismatch_error():
    with pytest.raises(DagsterInvalidDefinitionError, match='wrong pipeline name'):
        PipelineDefinition(
            name='pipeline_mismatch_test',
            solids=[
                SolidDefinition(
                    name='some_solid',
                    inputs=[],
                    outputs=[],
                    config_def=ConfigDefinition.solid_config_dict(
                        'wrong_pipeline',
                        'some_solid',
                        {},
                    ),
                    transform_fn=lambda *_args: None,
                ),
            ],
        )

    with pytest.raises(DagsterInvalidDefinitionError, match='wrong pipeline name'):
        PipelineDefinition(
            name='pipeline_mismatch_test',
            solids=[],
            context_definitions={
                'some_context':
                PipelineContextDefinition(
                    context_fn=lambda *_args: None,
                    config_def=ConfigDefinition.context_config_dict(
                        'not_a_match',
                        'some_context',
                        {},
                    )
                )
            }
        )


def test_solid_name_mismatch():
    with pytest.raises(DagsterInvalidDefinitionError, match='wrong solid name'):
        PipelineDefinition(
            name='solid_name_mismatch',
            solids=[
                SolidDefinition(
                    name='dont_match_me',
                    inputs=[],
                    outputs=[],
                    config_def=ConfigDefinition.solid_config_dict(
                        'solid_name_mismatch',
                        'nope',
                        {},
                    ),
                    transform_fn=lambda *_args: None,
                ),
            ],
        )

    with pytest.raises(DagsterInvalidDefinitionError, match='context_config_dict'):
        PipelineDefinition(
            name='solid_name_mismatch',
            solids=[
                SolidDefinition(
                    name='dont_match_me',
                    inputs=[],
                    outputs=[],
                    config_def=ConfigDefinition.context_config_dict(
                        'solid_name_mismatch',
                        'dont_match_me',
                        {},
                    ),
                    transform_fn=lambda *_args: None,
                ),
            ],
        )


def test_context_name_mismatch():
    with pytest.raises(DagsterInvalidDefinitionError, match='wrong context name'):
        PipelineDefinition(
            name='context_name_mismatch',
            solids=[],
            context_definitions={
                'test':
                PipelineContextDefinition(
                    context_fn=lambda *_args: None,
                    config_def=ConfigDefinition.context_config_dict(
                        'context_name_mismatch',
                        'nope',
                        {},
                    )
                )
            }
        )

    with pytest.raises(DagsterInvalidDefinitionError, match='solid_config_dict'):
        PipelineDefinition(
            name='context_name_mismatch',
            solids=[],
            context_definitions={
                'test':
                PipelineContextDefinition(
                    context_fn=lambda *_args: None,
                    config_def=ConfigDefinition.solid_config_dict(
                        'context_name_mismatch',
                        'some_solid',
                        {},
                    )
                )
            }
        )



from collections import OrderedDict
import uuid

from dagster.core.execution_context import (
    ExecutionContext,
    ReentrantContextInfo,
)

# pylint: disable=W0212


def test_noarg_ctor():
    context = ExecutionContext()
    assert context._context_stack == OrderedDict()


def test_for_run_ctor():
    context = ExecutionContext.for_run()
    assert uuid.UUID(context.get_context_value('run_id'))
    assert uuid.UUID(context.run_id)


def test_reentrant_info():
    context = ExecutionContext(reentrant_info=ReentrantContextInfo(OrderedDict()))
    assert context._context_stack == OrderedDict()


def test_reentrant_info_with_values():
    context = ExecutionContext(reentrant_info=ReentrantContextInfo(OrderedDict({'foo': 'bar'})))
    assert context._context_stack == OrderedDict({'foo': 'bar'})
    assert context.get_context_value('foo') == 'bar'



# pylint: disable=W0622,W0614,W0401

from dagster import (
    PipelineDefinition,
    config,
    execute_pipeline,
    solid,
)


@solid
def solid_one(info):
    info.context.info('Something you should know about occurred.')


@solid
def solid_two(info):
    info.context.error('An error occurred.')


def define_step_one_pipeline():
    return PipelineDefinition(name='part_five_step_one_pipeline', solids=[solid_one, solid_two])


def test_tutorial_part_five_sample_one():
    pipeline_result = execute_pipeline(define_step_one_pipeline())

    assert pipeline_result.success
    return pipeline_result


def define_step_two_pipeline():
    return PipelineDefinition(name='part_five_step_two_pipeline', solids=[solid_one, solid_two])


def test_tutorial_part_five_sample_two():
    pipeline_result = execute_pipeline(define_step_two_pipeline())

    assert pipeline_result.success
    return pipeline_result


def define_step_three_pipeline():
    return PipelineDefinition(name='part_five_step_three_pipeline', solids=[solid_one, solid_two])


def test_tutorial_part_five_sample_three():
    pipeline_result = execute_pipeline(
        define_step_two_pipeline(),
        config.Environment(context=config.Context(config={'log_level': 'DEBUG'}))
    )

    assert pipeline_result.success
    return pipeline_result


if __name__ == '__main__':
    # test_tutorial_part_five_sample_one()
    # test_tutorial_part_five_sample_two()
    test_tutorial_part_five_sample_three()



import json
import os

import jinja2

from dagster import (
    ConfigDefinition,
    Field,
    InputDefinition,
    OutputDefinition,
    Result,
    SolidDefinition,
    check,
    types,
)

from .common import execute_sql_text_on_context


class DagsterSqlTextType(types.DagsterStringType):
    def __init__(self):
        super(DagsterSqlTextType, self).__init__(
            name='SqlText',
            description='A string of SQL text that is directly executable.',
        )

    def serialize_value(self, output_dir, value):
        type_value = self.create_serializable_type_value(self.evaluate_value(value), output_dir)
        output_path = os.path.join(output_dir, 'type_value')
        with open(output_path, 'w') as ff:
            json.dump(
                {
                    'type': type_value.name,
                    'value': type_value.value,
                },
                ff,
            )

        with open(os.path.join(output_dir, 'sql'), 'w') as sf:
            sf.write(value)


SqlTextType = DagsterSqlTextType()


def create_templated_sql_transform_solid(name, sql, table_arguments, dependant_solids=None):
    check.str_param(name, 'name')
    check.str_param(sql, 'sql')
    check.list_param(table_arguments, 'table_arguments', of_type=str)

    dependant_solids = check.opt_list_param(
        dependant_solids, 'dependant_solids', of_type=SolidDefinition
    )

    field_dict = {}
    for table in table_arguments:
        field_dict[table] = Field(types.String)

    return SolidDefinition(
        name=name,
        inputs=[InputDefinition(solid.name) for solid in dependant_solids],
        config_def=ConfigDefinition.config_dict('{name}_Type'.format(name=name), field_dict),
        transform_fn=_create_templated_sql_transform_with_output(sql),
        outputs=[
            OutputDefinition(name='result', dagster_type=types.Any),
            OutputDefinition(name='sql_text', dagster_type=SqlTextType),
        ],
    )


def _render_template_string(template_text, config_dict):
    template = jinja2.Environment(loader=jinja2.BaseLoader).from_string(template_text)
    return template.render(**config_dict)


def _create_templated_sql_transform_with_output(sql):
    def do_transform(info, _inputs):
        rendered_sql = _render_template_string(sql, info.config)
        execute_sql_text_on_context(info.context, rendered_sql)
        yield Result(info.config)
        yield Result(output_name='sql_text', value=rendered_sql)

    return do_transform



# pylint: disable=W0622,W0614,W0401
import pytest

from dagster import (
    ConfigDefinition,
    DagsterInvariantViolationError,
    DependencyDefinition,
    InputDefinition,
    MultipleResults,
    OutputDefinition,
    PipelineDefinition,
    Result,
    config,
    execute_pipeline,
    solid,
    types,
)


@solid(
    outputs=[
        OutputDefinition(dagster_type=types.Int, name='out_one'),
        OutputDefinition(dagster_type=types.Int, name='out_two'),
    ],
)
def yield_outputs(_info):
    yield Result(23, 'out_one')
    yield Result(45, 'out_two')


@solid(
    outputs=[
        OutputDefinition(dagster_type=types.Int, name='out_one'),
        OutputDefinition(dagster_type=types.Int, name='out_two'),
    ],
)
def return_dict_results(_info):
    return MultipleResults.from_dict({
        'out_one': 23,
        'out_two': 45,
    })


@solid(
    config_def=ConfigDefinition(types.String, description='Should be either out_one or out_two'),
    outputs=[
        OutputDefinition(dagster_type=types.Int, name='out_one'),
        OutputDefinition(dagster_type=types.Int, name='out_two'),
    ],
)
def conditional(info):
    if info.config == 'out_one':
        yield Result(23, 'out_one')
    elif info.config == 'out_two':
        yield Result(45, 'out_two')
    else:
        raise Exception('invalid config')


@solid(inputs=[InputDefinition('num', dagster_type=types.Int)])
def log_num(info, num):
    info.context.info('num {num}'.format(num=num))
    return num


@solid(inputs=[InputDefinition('num', dagster_type=types.Int)])
def log_num_squared(info, num):
    info.context.info('num_squared {num_squared}'.format(num_squared=num * num))
    return num * num


def define_part_eleven_step_one_pipeline():
    return PipelineDefinition(
        name='part_eleven_step_one_pipeline',
        solids=[return_dict_results, log_num, log_num_squared],
        dependencies={
            'log_num': {
                'num': DependencyDefinition('return_dict_results', 'out_one')
            },
            'log_num_squared': {
                'num': DependencyDefinition('return_dict_results', 'out_two')
            },
        },
    )


def define_part_eleven_step_two_pipeline():
    return PipelineDefinition(
        name='part_eleven_step_two_pipeline',
        solids=[yield_outputs, log_num, log_num_squared],
        dependencies={
            'log_num': {
                'num': DependencyDefinition('yield_outputs', 'out_one')
            },
            'log_num_squared': {
                'num': DependencyDefinition('yield_outputs', 'out_two')
            },
        },
    )


def define_part_eleven_step_three_pipeline():
    return PipelineDefinition(
        name='part_eleven_step_three_pipeline',
        solids=[conditional, log_num, log_num_squared],
        dependencies={
            'log_num': {
                'num': DependencyDefinition('conditional', 'out_one')
            },
            'log_num_squared': {
                'num': DependencyDefinition('conditional', 'out_two')
            },
        },
    )


def test_intro_tutorial_part_eleven_step_one():
    result = execute_pipeline(define_part_eleven_step_one_pipeline())

    assert result.success
    assert result.result_for_solid('return_dict_results').transformed_value('out_one') == 23
    assert result.result_for_solid('return_dict_results').transformed_value('out_two') == 45
    assert result.result_for_solid('log_num').transformed_value() == 23
    assert result.result_for_solid('log_num_squared').transformed_value() == 45 * 45


def test_intro_tutorial_part_eleven_step_two():
    result = execute_pipeline(define_part_eleven_step_two_pipeline())

    assert result.success
    assert result.result_for_solid('yield_outputs').transformed_value('out_one') == 23
    assert result.result_for_solid('yield_outputs').transformed_value('out_two') == 45
    assert result.result_for_solid('log_num').transformed_value() == 23
    assert result.result_for_solid('log_num_squared').transformed_value() == 45 * 45


def test_intro_tutorial_part_eleven_step_three():
    result = execute_pipeline(
        define_part_eleven_step_three_pipeline(), {
            'solids': {
                'conditional': {
                    'config': 'out_two',
                },
            },
        }
    )

    # successful things
    assert result.success
    assert result.result_for_solid('conditional').transformed_value('out_two') == 45
    assert result.result_for_solid('log_num_squared').transformed_value() == 45 * 45

    # unsuccessful things
    with pytest.raises(DagsterInvariantViolationError):
        assert result.result_for_solid('conditional').transformed_value('out_one') == 45

    with pytest.raises(DagsterInvariantViolationError):
        result.result_for_solid('unnamed')

    with pytest.raises(DagsterInvariantViolationError):
        result.result_for_solid('log_num')


if __name__ == '__main__':
    execute_pipeline(
        define_part_eleven_step_three_pipeline(),
        {
            'solids': {
                'conditional': {
                    'config': 'out_two'
                },
            },
        },
    )

    # execute_pipeline(define_part_eleven_step_two())



import pytest

from dagster import (
    ConfigDefinition,
    DagsterEvaluateValueError,
    ExecutionContext,
    Field,
    PipelineContextDefinition,
    PipelineDefinition,
    SolidDefinition,
    config,
    types,
    execute_pipeline,
)

from dagster.core.config_types import (
    ContextConfigType,
    EnvironmentConfigType,
    ExecutionConfigType,
    ExpectationsConfigType,
    SolidConfigType,
    SolidDictionaryType,
    SpecificContextConfig,
    all_optional_user_config,
)


def test_context_config_any():
    context_defs = {
        'test':
        PipelineContextDefinition(
            config_def=ConfigDefinition(),
            context_fn=lambda *args: ExecutionContext(),
        )
    }

    context_config_type = ContextConfigType('something', context_defs)
    output = context_config_type.evaluate_value({'test': {'config': 1}})
    assert output.name == 'test'
    assert output.config == 1


def test_context_config():
    context_defs = {
        'test':
        PipelineContextDefinition(
            config_def=ConfigDefinition(
                config_type=types.
                ConfigDictionary('TestConfigDict', {
                    'some_str': Field(types.String),
                })
            ),
            context_fn=lambda *args: ExecutionContext(),
        )
    }

    context_config_type = ContextConfigType('something', context_defs)

    output = context_config_type.evaluate_value({'test': {'config': {'some_str': 'something'}}})
    assert isinstance(output, config.Context)
    assert output.name == 'test'
    assert output.config == {'some_str': 'something'}


def test_memoized_context():
    context_defs = {
        'test':
        PipelineContextDefinition(
            config_def=ConfigDefinition(),
            context_fn=lambda *args: ExecutionContext(),
        )
    }

    context_config_type = ContextConfigType('something', context_defs)

    output = context_config_type.evaluate_value(config.Context(name='test', config='whatever'))
    assert isinstance(output, config.Context)
    assert output.name == 'test'
    assert output.config == 'whatever'


def test_default_expectations():
    expect_config_type = ExpectationsConfigType('some_name')
    assert expect_config_type.evaluate_value({}).evaluate is True
    assert expect_config_type.evaluate_value(None).evaluate is True


def test_default_execution():
    execution_config_type = ExecutionConfigType('some_name')
    assert execution_config_type.evaluate_value({}).serialize_intermediates is False
    assert execution_config_type.evaluate_value(None).serialize_intermediates is False


def test_default_context_config():
    pipeline_def = PipelineDefinition(
        solids=[
            SolidDefinition(
                name='some_solid',
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            ),
        ],
    )

    context_config_type = ContextConfigType(pipeline_def.name, pipeline_def.context_definitions)
    assert 'default' in context_config_type.field_dict
    assert context_config_type.field_dict['default'].is_optional
    default_context_config_type = context_config_type.field_dict['default'].dagster_type

    assert isinstance(default_context_config_type, SpecificContextConfig)
    assert 'config' in default_context_config_type.field_dict

    assert all_optional_user_config(default_context_config_type)

    context_dict = context_config_type.evaluate_value({})

    assert 'default' in context_dict


def test_provided_default_config():
    pipeline_def = PipelineDefinition(
        context_definitions={
            'some_context':
            PipelineContextDefinition(
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        'ksjdkfjd', {
                            'with_default_int':
                            Field(
                                types.Int,
                                is_optional=True,
                                default_value=23434,
                            ),
                        }
                    )
                ),
                context_fn=lambda *args: None
            )
        },
        solids=[
            SolidDefinition(
                name='some_solid',
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            ),
        ],
    )

    env_type = EnvironmentConfigType(pipeline_def)
    env_obj = env_type.evaluate_value({})
    assert env_obj.context.name == 'some_context'


def test_default_environment():
    pipeline_def = PipelineDefinition(
        solids=[
            SolidDefinition(
                name='some_solid',
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            ),
        ],
    )

    env_type = EnvironmentConfigType(pipeline_def)
    env_obj = env_type.evaluate_value({})

    assert env_obj.expectations.evaluate is True
    assert env_obj.execution.serialize_intermediates is False


def test_errors():
    context_defs = {
        'test':
        PipelineContextDefinition(
            config_def=ConfigDefinition(
                types.ConfigDictionary(
                    'something',
                    {
                        'required_int': types.Field(types.Int),
                    },
                )
            ),
            context_fn=lambda *args: ExecutionContext(),
        )
    }

    context_config_type = ContextConfigType('something', context_defs)

    with pytest.raises(DagsterEvaluateValueError, match='must be None or dict'):
        context_config_type.evaluate_value(1)

    with pytest.raises(DagsterEvaluateValueError, match='Must specify in config'):
        context_config_type.evaluate_value({})

    # I tried doing some regular expressions here but the rules differ for escaping
    # between 2.7, 3.5, or 3.6 so gave up and did this hackneyed solution
    with pytest.raises(DagsterEvaluateValueError, match='You can only specify a single context'):
        context_config_type.evaluate_value({
            'context_one': 1,
            'context_two': 2,
        })

    with pytest.raises(DagsterEvaluateValueError, match="You specified"):
        context_config_type.evaluate_value({
            'context_one': 1,
            'context_two': 2,
        })

    with pytest.raises(DagsterEvaluateValueError, match="'context_one', 'context_two'"):
        context_config_type.evaluate_value({
            'context_one': 1,
            'context_two': 2,
        })

    with pytest.raises(DagsterEvaluateValueError, match="The available contexts are"):
        context_config_type.evaluate_value({
            'context_one': 1,
            'context_two': 2,
        })

    with pytest.raises(DagsterEvaluateValueError, match="'test'"):
        context_config_type.evaluate_value({
            'context_one': 1,
            'context_two': 2,
        })


def test_select_context():
    context_defs = {
        'int_context':
        PipelineContextDefinition(
            config_def=ConfigDefinition(types.Int),
            context_fn=lambda *args: ExecutionContext(),
        ),
        'string_context':
        PipelineContextDefinition(
            config_def=ConfigDefinition(types.String),
            context_fn=lambda *args: ExecutionContext(),
        ),
    }

    context_config_type = ContextConfigType('something', context_defs)

    assert context_config_type.evaluate_value({
        'int_context': {
            'config': 1
        },
    }) == config.Context(
        name='int_context',
        config=1,
    )

    assert context_config_type.evaluate_value({
        'string_context': {
            'config': 'bar'
        },
    }) == config.Context(
        name='string_context',
        config='bar',
    )

    # mismatched field type mismatch
    with pytest.raises(DagsterEvaluateValueError):
        assert context_config_type.evaluate_value({
            'int_context': {
                'config': 'bar'
            },
        })

    # mismatched field type mismatch
    with pytest.raises(DagsterEvaluateValueError):
        assert context_config_type.evaluate_value({
            'string_context': {
                'config': 1
            },
        })


def test_solid_config():
    solid_config_type = SolidConfigType('kdjfkd', types.Int)
    solid_inst = solid_config_type.evaluate_value({'config': 1})
    assert isinstance(solid_inst, config.Solid)
    assert solid_inst.config == 1


def test_expectations_config():
    expectations_config_type = ExpectationsConfigType('ksjdfkd')
    expectations = expectations_config_type.evaluate_value({'evaluate': True})

    assert isinstance(expectations, config.Expectations)
    assert expectations.evaluate is True

    assert expectations_config_type.evaluate_value({
        'evaluate': False
    }) == config.Expectations(evaluate=False)


def test_solid_dictionary_type():
    pipeline_def = define_test_solids_config_pipeline()

    solid_dict_type = SolidDictionaryType('foobar', pipeline_def)

    value = solid_dict_type.evaluate_value(
        {
            'int_config_solid': {
                'config': 1,
            },
            'string_config_solid': {
                'config': 'bar',
            },
        }
    )

    assert set(['int_config_solid', 'string_config_solid']) == set(value.keys())
    assert value == {
        'int_config_solid': config.Solid(1),
        'string_config_solid': config.Solid('bar'),
    }


def define_test_solids_config_pipeline():
    return PipelineDefinition(
        solids=[
            SolidDefinition(
                name='int_config_solid',
                config_def=ConfigDefinition(types.Int),
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            ),
            SolidDefinition(
                name='string_config_solid',
                config_def=ConfigDefinition(types.String),
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            )
        ]
    )


def test_solid_dictionary_some_no_config():
    pipeline_def = PipelineDefinition(
        solids=[
            SolidDefinition(
                name='int_config_solid',
                config_def=ConfigDefinition(types.Int),
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            ),
            SolidDefinition(
                name='no_config_solid',
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            )
        ]
    )

    solid_dict_type = SolidDictionaryType('foobar', pipeline_def)

    value = solid_dict_type.evaluate_value({
        'int_config_solid': {
            'config': 1,
        },
    })

    assert set(['int_config_solid']) == set(value.keys())
    assert value == {
        'int_config_solid': config.Solid(1),
    }


def test_whole_environment():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        context_definitions={
            'test':
            PipelineContextDefinition(
                config_def=ConfigDefinition(),
                context_fn=lambda *args: ExecutionContext(),
            )
        },
        solids=[
            SolidDefinition(
                name='int_config_solid',
                config_def=ConfigDefinition(types.Int),
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            ),
            SolidDefinition(
                name='no_config_solid',
                inputs=[],
                outputs=[],
                transform_fn=lambda *args: None,
            )
        ]
    )

    environment_type = EnvironmentConfigType(pipeline_def)

    assert environment_type.field_dict['context'].dagster_type.name == 'SomePipeline.ContextConfig'
    solids_type = environment_type.field_dict['solids'].dagster_type
    assert solids_type.name == 'SomePipeline.SolidsConfigDictionary'
    assert solids_type.field_dict['int_config_solid'
                                  ].dagster_type.name == 'SomePipeline.SolidConfig.IntConfigSolid'
    assert environment_type.field_dict['expectations'
                                       ].dagster_type.name == 'SomePipeline.ExpectationsConfig'

    env = environment_type.evaluate_value(
        {
            'context': {
                'test': {
                    'config': 1,
                }
            },
            'solids': {
                'int_config_solid': {
                    'config': 123,
                },
            },
        }
    )

    assert isinstance(env, config.Environment)
    assert env.context == config.Context('test', 1)
    assert env.solids == {
        'int_config_solid': config.Solid(123),
    }
    assert env.expectations == config.Expectations(evaluate=True)


def test_solid_config_error():
    solid_dict_type = SolidDictionaryType('slkdfjkjdsf', define_test_solids_config_pipeline())
    int_solid_config = solid_dict_type.field_dict['int_config_solid'].dagster_type

    with pytest.raises(DagsterEvaluateValueError, match='Field notconfig not found.'):
        int_solid_config.evaluate_value({'notconfig': 1})

    with pytest.raises(DagsterEvaluateValueError):
        int_solid_config.evaluate_value(1)


def test_execution_config():
    env_type = EnvironmentConfigType(define_test_solids_config_pipeline())
    env_obj = env_type.evaluate_value({'execution': {'serialize_intermediates': True}})
    assert isinstance(env_obj.execution, config.Execution)
    assert env_obj.execution.serialize_intermediates


def test_optional_solid_with_no_config():
    def _assert_config_none(info, value):
        assert info.config is value

    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[
            SolidDefinition(
                name='int_config_solid',
                config_def=ConfigDefinition(types.Int),
                inputs=[],
                outputs=[],
                transform_fn=lambda info, _inputs: _assert_config_none(info, 234),
            ),
            SolidDefinition(
                name='no_config_solid',
                inputs=[],
                outputs=[],
                transform_fn=lambda info, _inputs: _assert_config_none(info, None),
            )
        ]
    )

    env_type = EnvironmentConfigType(pipeline_def)
    env_obj = env_type.evaluate_value({'solids': {'int_config_solid': {'config': 234}}})

    assert env_obj.solids['int_config_solid'].config == 234

    assert execute_pipeline(pipeline_def, env_obj).success


def test_optional_solid_with_optional_scalar_config():
    def _assert_config_none(info, value):
        assert info.config is value

    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[
            SolidDefinition(
                name='int_config_solid',
                config_def=ConfigDefinition(types.Int),
                inputs=[],
                outputs=[],
                transform_fn=lambda info, _inputs: _assert_config_none(info, 234),
            ),
        ]
    )

    env_type = EnvironmentConfigType(pipeline_def)

    assert env_type.field_dict['solids'].is_optional is True

    solids_type = env_type.field_dict['solids'].dagster_type

    assert solids_type.field_dict['int_config_solid'].is_optional is True

    solids_default_obj = solids_type.evaluate_value({})

    assert solids_default_obj['int_config_solid'].config is None

    env_obj = env_type.evaluate_value({})

    assert env_obj.solids['int_config_solid'].config is None


def test_required_solid_with_required_subfield():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[
            SolidDefinition(
                name='int_config_solid',
                config_def=ConfigDefinition(
                    types.ConfigDictionary(
                        'TestRequiredSolidConfig',
                        {
                            'required_field': types.Field(types.String),
                        },
                    )
                ),
                inputs=[],
                outputs=[],
                transform_fn=lambda *_args: None,
            ),
        ]
    )

    env_type = EnvironmentConfigType(pipeline_def)

    assert env_type.field_dict['solids'].is_optional is False
    assert env_type.field_dict['solids'].dagster_type

    solids_type = env_type.field_dict['solids'].dagster_type
    assert solids_type.field_dict['int_config_solid'].is_optional is False
    int_config_solid_type = solids_type.field_dict['int_config_solid'].dagster_type
    assert int_config_solid_type.field_dict['config'].is_optional is False

    assert env_type.field_dict['context'].is_optional
    assert env_type.field_dict['execution'].is_optional
    assert env_type.field_dict['expectations'].is_optional

    env_obj = env_type.evaluate_value(
        {
            'solids': {
                'int_config_solid': {
                    'config': {
                        'required_field': 'foobar',
                    },
                },
            },
        },
    )

    assert env_obj.solids['int_config_solid'].config['required_field'] == 'foobar'

    with pytest.raises(DagsterEvaluateValueError):
        env_type.evaluate_value({'solids': {}})

    with pytest.raises(DagsterEvaluateValueError):
        env_type.evaluate_value({})


def test_optional_solid_with_optional_subfield():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[
            SolidDefinition(
                name='int_config_solid',
                config_def=ConfigDefinition(
                    types.ConfigDictionary(
                        'TestOptionalSolidConfig', {
                            'optional_field': types.Field(types.String, is_optional=True),
                        }
                    )
                ),
                inputs=[],
                outputs=[],
                transform_fn=lambda *_args: None,
            ),
        ]
    )

    env_type = EnvironmentConfigType(pipeline_def)
    assert env_type.field_dict['solids'].is_optional
    assert env_type.field_dict['context'].is_optional
    assert env_type.field_dict['execution'].is_optional
    assert env_type.field_dict['expectations'].is_optional


def test_required_context_with_required_subfield():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[],
        context_definitions={
            'some_context':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_context_config',
                        fields={
                            'required_field': types.Field(types.String),
                        },
                    ),
                ),
            ),
        },
    )

    env_type = EnvironmentConfigType(pipeline_def)
    assert env_type.field_dict['solids'].is_optional
    assert env_type.field_dict['context'].is_optional is False
    assert env_type.field_dict['execution'].is_optional
    assert env_type.field_dict['expectations'].is_optional

    context_union_config_type = env_type.field_dict['context'].dagster_type
    assert context_union_config_type.field_dict['some_context'].is_optional is False


def test_all_optional_field_on_single_context_dict():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[],
        context_definitions={
            'some_context':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_context_config',
                        fields={
                            'optional_field': types.Field(types.String, is_optional=True),
                        },
                    ),
                ),
            ),
        },
    )

    env_type = EnvironmentConfigType(pipeline_def)
    assert env_type.field_dict['solids'].is_optional
    assert env_type.field_dict['context'].is_optional
    assert env_type.field_dict['execution'].is_optional
    assert env_type.field_dict['expectations'].is_optional


def test_optional_and_required_context():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[],
        context_definitions={
            'optional_field_context':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_optional_context_config',
                        fields={
                            'optional_field': types.Field(types.String, is_optional=True),
                        },
                    ),
                ),
            ),
            'required_field_context':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_required_context_config',
                        fields={
                            'required_field': types.Field(types.String),
                        },
                    ),
                ),
            ),
        },
    )

    env_type = EnvironmentConfigType(pipeline_def)
    assert env_type.field_dict['solids'].is_optional
    assert env_type.field_dict['context'].is_optional is False
    context_type = env_type.field_dict['context'].dagster_type

    assert context_type.field_dict['optional_field_context'].is_optional
    assert context_type.field_dict['required_field_context'].is_optional

    assert env_type.field_dict['execution'].is_optional
    assert env_type.field_dict['expectations'].is_optional

    env_obj = env_type.evaluate_value(
        {
            'context': {
                'optional_field_context': {
                    'config': {
                        'optional_field': 'foobar',
                    },
                },
            },
        },
    )

    assert env_obj.context.name == 'optional_field_context'
    assert env_obj.context.config == {'optional_field': 'foobar'}


def test_default_optional_and_required_context():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[],
        context_definitions={
            'default':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_optional_context_config',
                        fields={
                            'optional_field': types.Field(types.String, is_optional=True),
                        },
                    ),
                ),
            ),
            'required_field_context':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_required_context_config',
                        fields={
                            'required_field': types.Field(types.String),
                        },
                    ),
                ),
            ),
        },
    )

    env_type = EnvironmentConfigType(pipeline_def)
    assert env_type.field_dict['context'].is_optional

    env_obj = env_type.evaluate_value({})
    assert env_obj.context.name == 'default'
    assert env_obj.context.config == {}


def test_default_optional_with_default_value_and_required_context():
    pipeline_def = PipelineDefinition(
        name='some_pipeline',
        solids=[],
        context_definitions={
            'default':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_optional_context_config',
                        fields={
                            'optional_field':
                            types.Field(
                                types.String,
                                is_optional=True,
                                default_value='foobar',
                            ),
                        },
                    ),
                ),
            ),
            'required_field_context':
            PipelineContextDefinition(
                context_fn=lambda *args: None,
                config_def=ConfigDefinition(
                    config_type=types.ConfigDictionary(
                        name='some_required_context_config',
                        fields={
                            'required_field': types.Field(types.String),
                        },
                    ),
                ),
            ),
        },
    )

    env_type = EnvironmentConfigType(pipeline_def)
    assert env_type.field_dict['context'].is_optional

    env_obj = env_type.evaluate_value({})
    assert env_obj.context.name == 'default'
    assert env_obj.context.config == {'optional_field': 'foobar'}



# pylint: disable=W0622,W0614,W0401
from logging import DEBUG

import pytest

from dagster import (
    ConfigDefinition,
    DagsterExpectationFailedError,
    DependencyDefinition,
    ExpectationDefinition,
    ExpectationResult,
    InputDefinition,
    OutputDefinition,
    PipelineDefinition,
    config,
    execute_pipeline,
    lambda_solid,
    solid,
    types,
)


@solid(
    config_def=ConfigDefinition(types.Int),
    outputs=[
        OutputDefinition(
            types.Int,
            expectations=[
                ExpectationDefinition(
                    name="check_positive",
                    expectation_fn=lambda _info, value: ExpectationResult(success=value > 0)
                )
            ]
        )
    ],
)
def injest_a(info):
    return info.config


@solid(
    config_def=ConfigDefinition(types.Int),
    outputs=[OutputDefinition(types.Int)],
)
def injest_b(info):
    return info.config


@lambda_solid(
    inputs=[InputDefinition('num_one', types.Int),
            InputDefinition('num_two', types.Int)],
    output=OutputDefinition(types.Int),
)
def add_ints(num_one, num_two):
    return num_one + num_two


def define_part_ten_step_one_pipeline():
    return PipelineDefinition(
        name='part_ten_step_one_pipeline',
        solids=[injest_a, injest_b, add_ints],
        dependencies={
            'add_ints': {
                'num_one': DependencyDefinition('injest_a'),
                'num_two': DependencyDefinition('injest_b'),
            },
        },
    )


def test_intro_tutorial_part_ten_step_one():
    result = execute_pipeline(
        define_part_ten_step_one_pipeline(),
        {
            'context': {
                'default': {
                    'config': {
                        'log_level': 'DEBUG',
                    }
                }
            },
            'solids': {
                'injest_a': {
                    'config': 2,
                },
                'injest_b': {
                    'config': 3,
                },
            }
        },
    )

    assert result.success


def define_failing_environment_config():
    return {
        'context': {
            'default': {
                'config': {
                    'log_level': 'DEBUG',
                }
            }
        },
        'solids': {
            'injest_a': {
                'config': -2,
            },
            'injest_b': {
                'config': 3,
            },
        }
    }


def test_intro_tutorial_part_ten_step_two_fails_hard():
    with pytest.raises(DagsterExpectationFailedError):
        execute_pipeline(
            define_part_ten_step_one_pipeline(),
            define_failing_environment_config(),
        )


def test_intro_tutorial_part_ten_step_two_fails_soft():
    result = execute_pipeline(
        define_part_ten_step_one_pipeline(),
        define_failing_environment_config(),
        throw_on_error=False,
    )

    assert not result.success


if __name__ == '__main__':
    execute_pipeline(
        define_part_ten_step_one_pipeline(),
        {
            'context': {
                'default': {
                    'config': {
                        'log_level': 'DEBUG',
                    }
                }
            },
            'solids': {
                'injest_a': {
                    'config': -2,
                },
                'injest_b': {
                    'config': 3,
                },
            },
            'expectations': {
                'evaluate': True,
            },
        },
    )



from dagster import (
    PipelineContextDefinition,
    PipelineDefinition,
    SolidDefinition,
    ConfigDefinition,
    check,
    types,
)

from dagster.core.config_types import EnvironmentConfigType

from dagster.cli.config_scaffolder import (
    scaffold_pipeline_config,
    scaffold_type,
)


def fail_me():
    return check.failed('Should not call')


def test_scalars():
    assert scaffold_type(types.Int) == 0
    assert scaffold_type(types.String) == ''
    assert scaffold_type(types.Path) == 'path/to/something'
    assert scaffold_type(types.Bool) is True
    assert scaffold_type(types.Dict) == {}
    assert scaffold_type(types.Any) == 'AnyType'


def test_basic_solids_config():
    pipeline_def = PipelineDefinition(
        name='BasicSolidsConfigPipeline',
        solids=[
            SolidDefinition(
                name='required_field_solid',
                inputs=[],
                outputs=[],
                config_def=ConfigDefinition(
                    types.ConfigDictionary(
                        'RequiredFieldSolidConfigDict',
                        {'required_int': types.Field(types.Int)},
                    )
                ),
                transform_fn=lambda *_args: fail_me(),
            )
        ]
    )

    env_config_type = EnvironmentConfigType(pipeline_def)

    assert env_config_type.field_dict['solids'].is_optional is False
    solids_config_type = env_config_type.field_dict['solids'].dagster_type
    assert solids_config_type.field_dict['required_field_solid'].is_optional is False
    required_solid_config_type = solids_config_type.field_dict['required_field_solid'].dagster_type
    assert required_solid_config_type.field_dict['config'].is_optional is False

    context_config_type = env_config_type.field_dict['context'].dagster_type

    assert 'default' in context_config_type.field_dict
    assert context_config_type.field_dict['default'].is_optional

    default_context_config_type = context_config_type.field_dict['default'].dagster_type

    assert set(default_context_config_type.field_dict.keys()) == set(['config'])

    default_context_user_config_type = default_context_config_type.field_dict['config'].dagster_type

    assert set(default_context_user_config_type.field_dict.keys()) == set(['log_level'])

    assert scaffold_pipeline_config(
        pipeline_def,
        skip_optional=False,
    ) == {
        'context': {
            'default': {
                'config': {
                    'log_level': '',
                },
            },
        },
        'solids': {
            'required_field_solid': {
                'config': {
                    'required_int': 0,
                },
            },
        },
        'execution': {
            'serialize_intermediates': True,
        },
        'expectations': {
            'evaluate': True,
        },
    }


def test_two_contexts():
    pipeline_def = PipelineDefinition(
        name='TwoContextsPipeline',
        solids=[],
        context_definitions={
            'context_one':
            PipelineContextDefinition(
                context_fn=lambda *args: fail_me(),
                config_def=ConfigDefinition(
                    types.ConfigDictionary(
                        'ContextOneConfigDict',
                        {
                            'context_one_field': types.Field(types.String),
                        },
                    ),
                ),
            ),
            'context_two':
            PipelineContextDefinition(
                context_fn=lambda *args: fail_me(),
                config_def=ConfigDefinition(
                    types.ConfigDictionary(
                        'ContextTwoConfigDict',
                        {
                            'context_two_field': types.Field(types.Int),
                        },
                    ),
                ),
            ),
        },
    )

    assert scaffold_pipeline_config(pipeline_def) == {'context': {}}

    assert scaffold_pipeline_config(
        pipeline_def, skip_optional=False
    ) == {
        'context': {
            'context_one': {
                'config': {
                    'context_one_field': '',
                },
            },
            'context_two': {
                'config': {
                    'context_two_field': 0,
                },
            },
        },
        'solids': {},
        'execution': {
            'serialize_intermediates': True,
        },
        'expectations': {
            'evaluate': True,
        },
    }



# pylint: disable=W0622,W0614,W0401
from dagster import (
    ConfigDefinition,
    PipelineDefinition,
    config,
    execute_pipeline,
    solid,
    types,
)


@solid(config_def=ConfigDefinition(types.String))
def hello_world(info):
    print(info.config)
    return info.config


def define_pipeline():
    return PipelineDefinition(name='part_four_pipeline', solids=[hello_world])


def test_tutorial_part_four():

    result = execute_pipeline(
        define_pipeline(),
        {
            'solids': {
                'hello_world': {
                    'config': 'Hello, World!',
                },
            },
        },
    )

    assert result.success
    assert len(result.result_list) == 1
    assert result.result_for_solid('hello_world').transformed_value() is 'Hello, World!'
    return result


if __name__ == '__main__':
    test_tutorial_part_four()
